@@ -111,6 +111,9 @@ class Peak:
         """
         determine if the peak position (and some neighborhood around it) is in empty region of mask
         """
+        # TODO:
+        #  the ordering of the dimensions in the Peak may not be consistent with ordering in Mask
+        #  this info would be available from PeakTable.axis_property
         try:
             return mask.ppm_in_mask(
                 peak_ppm=self.get_par('*_PPM'),
@@ -126,13 +129,13 @@ class PeakTable:
     A class for containing a collection of peaks (e.g. from a peak picker or synthetic peak generator)
     Sub-classes are defined for each peak table file format.
     """
-    def __init__(self, file):
+    def __init__(self, file, **kwargs):
         """
         Populate a peak list.  Each type of peak list file is sub-classed from PeakTable.  The subclasses should have
         an __init__ that defines "pre" operations, calls super.__init__, defines "post" operations.  The __init__ here
         includes pre operations for ALL PeakTable subclasses and it includes a call to _parse_file (which should be
         defined within each subclass).  The subclass __init__ should then call _post to run operations that are defined
-        for ALL PeakTable subclasses.  By crating an instance of a subclass, the following operations are performed:
+        for ALL PeakTable subclasses.  By creating an instance of a subclass, the following operations are performed:
             "pre" operations in subclass __init__
             "pre" operations in PeakTable __init__
             _parse_file (defined by subclass)
@@ -147,9 +150,10 @@ class PeakTable:
                 # pre
                 <define instance variables used specifically by this subclass>
 
-                # perform operations before the table file is parsed that are for ALL subclasses
-                # call the _parse_file method that is defined by the subclass
-                super().__init__(file)
+                # call PeakTable.__init__ which includes:
+                #   (1) "pre" operations for all subclasses
+                #   (2) _parse_file method, which is defined by the subclass
+                super().__init__(file, **kwargs)
 
                 # post
                 <perform operations after table file is parsed that are specific to this subclass>
@@ -173,15 +177,22 @@ class PeakTable:
         # global properties that describe the collection of peaks
         self.table_property = TableData()
 
-        # subclasses will each call _parse_file() to populate self.peaks and self.axis
+        # subclasses will each call _parse_file() to populate self.peaks and self.axis_property
         # subclasses can optionally include "pre" and "post" tasks in their __init__ to perform before/after parsing
 
         try:
             self._parse_file()
+        except NotImplementedError as e:
+            raise ParsePeakTableError(e)
         except ValueError as e:
             raise ParsePeakTableError(f'failed to read peak table: {self.file}\n{e}')
-        except:
-            raise ParsePeakTableError(f'failed to read peak table: {self.file}')
+        except Exception as e:
+            raise ParsePeakTableError(f'failed to read peak table: {self.file}\n{e}')
+
+        try:
+            self._set_carrier_frequency(**kwargs)
+        except Exception as e:
+            raise ParsePeakTableError(f'failed to set carrier frequency: {self.file}\n{e}')
 
     def _parse_file(self):
         """stub - subclasses must define this and populate self.peaks and self.axis"""
@@ -189,17 +200,6 @@ class PeakTable:
 
     def _post(self):
         """operations to run after an instance of any subclass has been created"""
-        try:
-            # includes carrier frequency (i.e. PeakTablePipeInj)
-            self.axis_property.get('carrier_frequency')
-        except KeyError:
-            try:
-                # does NOT include carrier frequency (i.e. PeakTablePipeRec)
-                # try to compute carrier frequency by using peak positions in ppm and Hz
-                self._set_carrier_frequency_from_ppm_hz()
-            except KeyError:
-                print('not able to set carrier frequency automatically')
-                pass
         try:
             self.get_par(par='*_HZ')
         except KeyError:
@@ -209,6 +209,50 @@ class PeakTable:
                 print('not able to compute peaks positions in Hz automatically')
                 pass
 
+    def _set_carrier_frequency(
+            self, carrier_frequency=None, carrier_frequency_unit='MHz', **kwargs):
+        # the injected peaks are presumed to be inserted into an empirical data set, so read the
+        # carrier frequency out of the empirical peak table, because it is not included in the
+        # injected peak table
+        # carrier_frequency = self.empirical_table.axis_property.get(name='carrier_frequency')
+
+        if carrier_frequency is None:
+            # try to auto-determine carrier frequency from ppm and Hz values
+            try:
+                self._set_carrier_frequency_from_ppm_hz()
+            except KeyError as e:
+                raise KeyError(e)
+
+        elif isinstance(carrier_frequency, AxisData):
+            if set(carrier_frequency.axis_list()) != set(self.axis_property.axis_list()):
+                raise AttributeError('The axes for the given carrier frequency data dot match those of the peak table')
+            self.axis_property.set_data(carrier_frequency)
+
+        else:
+            # expect numerical data
+            try:
+                cf_values = np.asarray(carrier_frequency).astype(float, casting='safe')
+            except (ValueError, TypeError):
+                raise ValueError('unable to convert carrier frequency data')
+
+            if cf_values.size != self.num_dims():
+                raise AttributeError('number of carrier frequencies does not match number of peak table dimensions')
+
+            if carrier_frequency_unit == 'Hz':
+                cf_hz = cf_values
+            elif carrier_frequency_unit == 'MHz':
+                cf_hz = cf_values * 1e6
+            else:
+                raise ValueError(f'do not recognize unit for carrier frequency: {carrier_frequency_unit}')
+
+            for ax, cf in zip(self.axis_list(), cf_hz):
+                self.axis_property.set(
+                    axis=ax,
+                    name='carrier_frequency',
+                    value=cf,
+                    unit='Hz',
+                )
+
     def _set_carrier_frequency_from_ppm_hz(self):
         """Determine carrier frequency from the ratio between peak positions in Hz and ppm"""
         # get average hz/ppm ratio across all peaks for each dimension
@@ -216,7 +260,14 @@ class PeakTable:
         try:
             ppm = self.get_par(par='*_PPM')
             hz = self.get_par(par='*_HZ')
-            carrier = np.mean(hz / ppm, axis=0) * 1e6
+
+            # minimum ppm magnitude to use for computing carrier (avoid divide by 0 or even divide by small value)
+            ppm_min = 1
+
+            carrier = 1e6 * np.mean(
+                np.asarray([hz_row/ppm_row for hz_row, ppm_row in zip(hz, ppm) if (np.abs(ppm_row) > ppm_min).all()]),
+                axis=0,
+            )
         except KeyError as e:
             raise KeyError(e)
 
@@ -237,16 +288,14 @@ class PeakTable:
             except KeyError:
                 raise KeyError(f'missing peak PPM data for axis: {axis}')
             try:
-                cf = self.axis_property.get(
-                    name='carrier_frequency',
-                    axis=axis,
-                )[0]
-                if cf.unit == 'Hz':
-                    cf_mhz = cf.value / 1e6
-                elif cf.unit == 'MHz':
-                    cf_mhz = cf.value
+                unit = self.axis_property.get_field(name='carrier_frequency', axis=axis, field='unit')[0]
+                value = self.axis_property.get_field(name='carrier_frequency', axis=axis, field='value')[0]
+                if unit == 'Hz':
+                    cf_mhz = value / 1e6
+                elif unit == 'MHz':
+                    cf_mhz = value
                 else:
-                    raise ValueError(f'do not recognize unit for carrier frequency: {cf.unit}')
+                    raise ValueError(f'do not recognize unit for carrier frequency: {unit}')
                 peak_hz = peak_ppm * cf_mhz
             except KeyError:
                 raise KeyError(f'missing carrier frequency for axis: {axis}')
@@ -372,9 +421,13 @@ class PeakTable:
         """Get the axis names"""
         return list(self.axis_property.axis_list())
 
+    def num_dims(self):
+        """Number of dimensions"""
+        return len(self.axis_list())
+
     def label_list(self):
         """Get the axis labels"""
-        return self.axis_property.get_value(name='label')
+        return self.axis_property.get_field(name='label')
 
     def axis_label_list(self):
         """Get a list of tuples where each contains an axis and its label"""
@@ -411,7 +464,7 @@ class PeakTable:
 
 
 class PeakTablePipe(PeakTable):
-    def __init__(self, file):
+    def __init__(self, file, **kwargs):
         """
         This class has subclasses for peak tables coming from INJECTED and RECOVERED peak tables; use those subclasses.
         You probably don't want to create a PeakTablePipe object - it is just a super-class to provide common tasks.
@@ -420,7 +473,7 @@ class PeakTablePipe(PeakTable):
         self.VARS = None
         self.FORMAT = None
 
-        super().__init__(file)
+        super().__init__(file, **kwargs)
 
     def _parse_file(self):
         """
@@ -531,12 +584,12 @@ class PeakTablePipe(PeakTable):
 
 class PeakTablePipeRec(PeakTablePipe):
     """This subclass is for capturing the peaks "recovered" from an experiment by the NMRPipe peak picker"""
-    def __init__(self, file):
+    def __init__(self, file, **kwargs):
         # pre
         self.REMARK = []
 
         try:
-            super().__init__(file)
+            super().__init__(file, **kwargs)
         except:
             raise ParsePeakTableError(f'failed to read peak table as type: PeakTablePipeRec\n{file}')
 
@@ -594,37 +647,23 @@ class PeakTablePipeRec(PeakTablePipe):
 
 class PeakTablePipeInj(PeakTablePipe):
     """This subclass is for capturing the synthetic peaks "injected" by the NMRPipe genSimTab.tcl tool"""
-    def __init__(self, file, carrier_frequency):
+    def __init__(self, file, **kwargs):
+        """
+        :param carrier_frequency: either a list of values in MHz or an AxisData object (e.g. from another PeakTable)
+        """
         # pre
         self.genSimTab = []
-        self.carrier_frequency = carrier_frequency
 
         try:
-            super().__init__(file)
+            super().__init__(file, **kwargs)
         except:
             raise ParsePeakTableError(f'failed to read peak table as type: PeakTablePipeInj\n{file}')
 
         # post
         self.genSimTab = ' '.join(self.genSimTab)
         self._set_maxLW()
-        self._set_carrier_frequency()
         self._post()
 
-    def _set_carrier_frequency(self):
-        # the injected peaks are presumed to be inserted into an empirical data set, so read the
-        # carrier frequency out of the empirical peak table, because it is not included in the
-        # injected peak table
-        # carrier_frequency = self.empirical_table.axis_property.get(name='carrier_frequency')
-
-        # TODO: not great - this is using an implicitly ordered list to set values
-        for ax, cf in zip(self.axis_list(), self.carrier_frequency):
-            self.axis_property.set(
-                axis=ax,
-                name='carrier_frequency',
-                value=cf.value,
-                unit=cf.unit,
-            )
-
     def _set_maxLW(self):
         # parse through the tokens to find maxLW values for each axis
         tokens = self.genSimTab.split()
