@@ -117,7 +117,7 @@ class roc:
         # Get the max LW values used by genSimTab and then expand by a lw_scalar to define the distance along each axis
         # used to define if a recovered peak is within the accepted range of an injected peak to be considered a match.
         self.lw_scalar = lw_scalar
-        self.peak_range_hz = np.array(self.injected_peaks.axis_property.get_value('maxLW') * self.lw_scalar)
+        self.peak_range_hz = np.array(self.injected_peaks.axis_property.get_field('maxLW')) * self.lw_scalar
 
         # ================================================
         # run the reduce commands (if any parameters are not given to roc, then they are None and reduce skips them)
@@ -192,19 +192,22 @@ class roc:
         recovered_peaks_hz = self.recovered_peaks.get_par(par="*_HZ")
         recovered_peaks_ppm = self.recovered_peaks.get_par(par="*_PPM")
 
-        # compute pairwise distances from between recovered and injected peaks - to be used for defining matches
-        # TODO:
+        # compute pairwise distances between recovered and injected peaks - to be used for defining matches
         #  Currently finding closest injected peak to each recovered peak with unweighted L2, but at least it is in
         #  Hz and not ppm.  Eventually, the closest match identified here is more accurately assessed below by taking
-        #  the distance along along each axis and comparing to a cutoff distance defined as multiple of linewidth.
-        #  So... should a weighted L2 be sued here?
-        # start_time = time.time()
+        #  the distance along each axis and comparing to a cutoff distance defined as multiple of linewidth.
+        #  Q: Is it possible that unweighted L2 matches incorrect injected peak to a recovered peak?
+        #  A: Maybe.  But...
+        #       - it's probably a recovered peak that isn't even near an injected peak
+        #       - running several recovered peak tables through this shows that both methods yield same result
+        #       - compute times are roughly 100:1 ratio (weighted vs unweighted) but on the order of 5s vs .05s
+        #  Conclusion: just use unweighted L2 for now - can always toggle over to weighted L2, but must define weights
         pairwise_dist = distance_matrix(recovered_peaks_hz, injected_peaks_hz)
-        # print(f"scipy time: {time.time() - start_time}")
-
-        # start_time = time.time()
-        # pairwise_dist2 = pairwise_weighted_norm(recovered_peaks_hz, injected_peaks_hz, w=np.array((1, 1, 1)))
-        # print(f"custom L2 time: {time.time() - start_time}")
+        # pairwise_dist = pairwise_weighted_norm(
+        #     A=recovered_peaks_hz,
+        #     B=injected_peaks_hz,
+        #     w=np.array([1] * self.recovered_peaks.num_dims()),
+        # )
 
         # for each recovered peak find the closest injected peak
         idx_recovered2injected = np.argmin(pairwise_dist, axis=1)
